/*-----------------------------------------------------------------------------
/ Title      : iMS Test Program FileSystem Sub-Menu Source
/ Project    : Isomet Modular Synthesiser System
/------------------------------------------------------------------------------
/ File       : $URL: http://nutmeg/svn/sw/trunk/09-Isomet/iMS_SDK/console_app/SubMenus/SingleToneSubMenu.cpp $
/ Author     : $Author: dave $
/ Company    : Isomet (UK) Ltd
/ Created    : 2017-09-06
/ Last update: $Date: 2017-09-11 23:55:34 +0100 (Mon, 11 Sep 2017) $
/ Platform   :
/ Standard   : C++11
/ Revision   : $Rev: 300 $
/------------------------------------------------------------------------------
/ Description:
/------------------------------------------------------------------------------
/ Copyright (c) 2017 Isomet (UK) Ltd. All Rights Reserved.
/------------------------------------------------------------------------------
/ Revisions  :
/ Date        Version  Author  Description
/ 2017-09-06  1.0      dc      Created
/
/----------------------------------------------------------------------------*/

#include "SingleToneSubMenu.h"

#include <iostream>
#include <sstream>
#include <fstream>

static std::string desc("Use Single Tone Mode");

const int SingleToneSubMenu::num_submenu_items = 8;

SingleToneSubMenu::SingleToneSubMenu(iMS::IMSSystem& ims) :
		thisIMS(ims),
		sp(new iMS::SignalPath(ims))
{

}

SingleToneSubMenu::~SingleToneSubMenu() {
	delete sp;
}

const std::string& SingleToneSubMenu::SubMenuDescription() const
{
	return desc;
}

void SingleToneSubMenu::SubMenuPrint()
{
	std::cout << std::endl << std::endl;
	std::cout << " ******** SINGLE TONE MODE MENU ********" << std::endl;
	std::cout << " 1) Apply Single Tone" << std::endl;
	std::cout << " 2) Clear Single Tone" << std::endl;
	std::cout << " q) Back to Main Menu" << std::endl;
}

void SingleToneSubMenu::LaunchSubMenu()
{
	std::string s;
	double d;

	do {
		this->SubMenuPrint();
		std::getline( std::cin, s);
		if ((s.find("q") != std::string::npos) || (s.find("Q") != std::string::npos)) {
			break;
		} else {
			int index;
			std::istringstream iss(s);
			iss >> index;
			if (!iss.fail()) {
				if ((index > 0) && (index <= num_submenu_items)) {
					switch(index) {
					case 1: {
						std::cout << "Enter Frequency(MHz): ";
						std::getline( std::cin, s);
						iss.clear();
						iss.str(s);
						iss >> d;
						if (!iss.fail()) {
							fap.freq = d;
							std::cout << "Enter Amplitude(%): ";
							std::getline( std::cin, s);
							iss.clear();
							iss.str(s);
							iss >> d;
							if (!iss.fail()) {
								fap.ampl = d;
								std::cout << "Enter Phase Offset per Channel(deg): ";
								std::getline( std::cin, s);
								iss.clear();
								iss.str(s);
								iss >> d;
								if (!iss.fail()) {
									fap.phase = d;
									sp->SetCalibrationTone(fap);
								} else {
									std::cout << "Invalid Phase" << std::endl;
								}
							} else {
								std::cout << "Invalid Amplitude" << std::endl;
							}
						} else {
							std::cout << "Invalid Frequency" << std::endl;
						}
					}
					break;
					case 2: {
						// Set output to zero amplitude first ...
						fap.ampl = 0.0;
						sp->SetCalibrationTone(fap);

						// .. Then exit calibration tone mode
						sp->ClearTone();
						std::cout << "Stopped Calibration Tone output" << std::endl;
					}
					break;
					}
				}
			}
		}
	} while(1);

}



