/*-----------------------------------------------------------------------------
/ Title      : iMS Test Program FileSystem Sub-Menu Source
/ Project    : Isomet Modular Synthesiser System
/------------------------------------------------------------------------------
/ File       : $URL: http://nutmeg/svn/sw/trunk/09-Isomet/iMS_SDK/console_app/SubMenus/FilesystemSubMenu.cpp $
/ Author     : $Author: dave $
/ Company    : Isomet (UK) Ltd
/ Created    : 2017-09-06
/ Last update: $Date: 2017-09-11 23:55:34 +0100 (Mon, 11 Sep 2017) $
/ Platform   :
/ Standard   : C++11
/ Revision   : $Rev: 300 $
/------------------------------------------------------------------------------
/ Description:
/------------------------------------------------------------------------------
/ Copyright (c) 2017 Isomet (UK) Ltd. All Rights Reserved.
/------------------------------------------------------------------------------
/ Revisions  :
/ Date        Version  Author  Description
/ 2017-09-06  1.0      dc      Created
/
/----------------------------------------------------------------------------*/

#include "FilesystemSubMenu.h"

#include <iostream>
#include <sstream>
#include <fstream>

static std::string desc("File System Operations");

extern void ClearScreen();

const int FilesystemSubMenu::num_submenu_items = 8;

FilesystemSubMenu::FilesystemSubMenu(iMS::IMSSystem& ims) :
		thisIMS(ims),
		fstv(new iMS::FileSystemTableViewer(ims)),
		fsm(new iMS::FileSystemManager(ims))
{

}

FilesystemSubMenu::~FilesystemSubMenu() {
	delete fstv;
	delete fsm;
}

const std::string& FilesystemSubMenu::SubMenuDescription() const
{
	return desc;
}

void FilesystemSubMenu::SubMenuPrint()
{
	std::cout << std::endl << std::endl;
	std::cout << " ******** FILE SYSTEMS OPERATION MENU ********" << std::endl;
	std::cout << " 1) Display File System Table" << std::endl;
	std::cout << " 2) Send local file to File System" << std::endl;
	std::cout << " 3) Retrieve File Entry" << std::endl;
	std::cout << " 4) Delete File Entry" << std::endl;
	std::cout << " 5) Set Default Flag (runs at startup)" << std::endl;
	std::cout << " 6) Clear Default Flag" << std::endl;
	std::cout << " 7) Sanitize File System" << std::endl;
	std::cout << " 8) Execute File Entry" << std::endl;
	std::cout << " q) Back to Main Menu" << std::endl;
}

void FilesystemSubMenu::LaunchSubMenu()
{
	std::string s;
	do {
		this->SubMenuPrint();
		std::getline( std::cin, s);
		if ((s.find("q") != std::string::npos) || (s.find("Q") != std::string::npos)) {
			break;
		} else {
			int index;
			std::istringstream iss(s);
			iss >> index;
			if (!iss.fail()) {
				if ((index > 0) && (index <= num_submenu_items)) {
					switch(index) {
					case 1: {
						// Display File Table
						ClearScreen();
						if (!fstv->IsValid())
							std::cout << "No Filesystem found" << std::endl;
						else
							std::cout << *fstv ;
					}
					break;
					case 2: {
						// Send Local File
						std::cout << "Name of local file to send: ";
						std::getline( std::cin, s);

						// open the file:
						std::ifstream file(s.c_str(), std::ios::binary);

						if (file.good()) {
							// Stop eating new lines in binary mode!!!
							file.unsetf(std::ios::skipws);

							// get its size:
							std::streampos fileSize;

							file.seekg(0, std::ios::end);
							fileSize = file.tellg();
							file.seekg(0, std::ios::beg);

							// reserve capacity
							std::vector<std::uint8_t> vec;
							vec.reserve(fileSize);

							// read the data:
							vec.insert(vec.begin(),
									std::istream_iterator<std::uint8_t>(file),
									std::istream_iterator<std::uint8_t>());

							iMS::UserFileWriter ufw(thisIMS, vec, s);
							int fsi = ufw.Program();
							std::cout << "Completed file download. New file's index is: " << fsi << std::endl;
						} else {
							std::cout << "Unable to open file \"" << s << "\"" << std::endl;
						}
						file.close();
					}
					break;
					case 3: {
						std::cout << "Retrieve which file entry?";
						std::getline( std::cin, s);
						int fsidx;
						std::istringstream iss(s);
						iss >> fsidx;
						if (!iss.fail()) {
							if ((*fstv)[fsidx].Type() == iMS::FileSystemTypes::USER_DATA)
							{
								std::vector<uint8_t> readback_data;
								iMS::UserFileReader ufr(thisIMS, fsidx);
								ufr.Readback(readback_data);

								std::ofstream ufrFile ((*fstv)[fsidx].Name(), ios::out | ios::binary);
								if (ufrFile.good()) {
									ufrFile.write ((char*)&readback_data[0], readback_data.size());
								}
								ufrFile.close();
							}
							else
							{
								std::cout << "This file type doesn't support user readback" << std::endl;
							}
						}
					}
					break;
					case 4: {
						// Delete File Entry
						std::cout << "Delete which file entry?";
						std::getline( std::cin, s);
						int fsidx;
						std::istringstream iss(s);
						iss >> fsidx;
						if (!iss.fail()) {
							if (fsm->Delete(fsidx)) {
								std::cout << "Deleted File Entry " << fsidx << std::endl;
							} else {
								std::cout << "Delete operation failed" << std::endl;
							}
						}
					}
					break;
					case 5: {
						// Set Default Flag
						std::cout << "Set Default Flag on which file entry?";
						std::getline( std::cin, s);
						int fsidx;
						std::istringstream iss(s);
						iss >> fsidx;
						if (!iss.fail()) {
							if (fsm->SetDefault(fsidx)) {
								std::cout << "File Entry " << fsidx << " marked as default" << std::endl;
							} else {
								std::cout << "Set Default operation failed" << std::endl;
							}
						}
					}
					break;
					case 6: {
						// Clear Default Flag
						std::cout << "Clear Default Flag on which file entry?";
						std::getline( std::cin, s);
						int fsidx;
						std::istringstream iss(s);
						iss >> fsidx;
						if (!iss.fail()) {
							if (fsm->ClearDefault(fsidx)) {
								std::cout << "File Entry " << fsidx << " not marked as default" << std::endl;
							} else {
								std::cout << "Clear Default operation failed" << std::endl;
							}
						}
					}
					break;
					case 7: {
						// Sanitize File System
						if (!fstv->IsValid())
							std::cout << "No Filesystem available" << std::endl;
						else {
							if (fsm->Sanitize()) {
								ClearScreen();
								std::cout << "File System Sanitized" << std::endl;
								std::cout << *fstv ;
							} else {
								std::cout << "Sanitize operation failed" << std::endl;
							}
						}
					}
					break;
					case 8: {
						// Execute File Entry
						std::cout << "Execute which file entry?";
						std::getline( std::cin, s);
						int fsidx;
						std::istringstream iss(s);
						iss >> fsidx;
						if (!iss.fail()) {
							if (fsm->Execute(fsidx)) {
								std::cout << "File Entry " << fsidx << " executed successfully" << std::endl;
							} else {
								std::cout << "Execute operation failed" << std::endl;
							}
						}
					}
					break;
					}
				}
			}
		}
	} while(1);

}
