/*-----------------------------------------------------------------------------
/ Title      : iMS Test Program FileSystem Sub-Menu Source
/ Project    : Isomet Modular Synthesiser System
/------------------------------------------------------------------------------
/ File       : $URL: http://nutmeg/svn/sw/trunk/09-Isomet/iMS_SDK/console_app/SubMenus/SignalPathControlSubMenu.cpp $
/ Author     : $Author: dave $
/ Company    : Isomet (UK) Ltd
/ Created    : 2017-09-06
/ Last update: $Date: 2017-09-11 23:55:34 +0100 (Mon, 11 Sep 2017) $
/ Platform   :
/ Standard   : C++11
/ Revision   : $Rev: 300 $
/------------------------------------------------------------------------------
/ Description:
/------------------------------------------------------------------------------
/ Copyright (c) 2017 Isomet (UK) Ltd. All Rights Reserved.
/------------------------------------------------------------------------------
/ Revisions  :
/ Date        Version  Author  Description
/ 2017-09-06  1.0      dc      Created
/
/----------------------------------------------------------------------------*/

#include "SignalPathControlSubMenu.h"

#include <iostream>
#include <sstream>
#include <fstream>

static std::string desc("RF Signal Path Control");

const int SignalPathControlSubMenu::num_submenu_items = 8;

SignalPathControlSubMenu::SignalPathControlSubMenu(iMS::IMSSystem& ims) :
		thisIMS(ims),
		sp(new iMS::SignalPath(ims))
{
	this->amplComp = iMS::SignalPath::Compensation::ACTIVE;
	this->phaseComp = iMS::SignalPath::Compensation::ACTIVE;
	this->XYMode = false;
}

SignalPathControlSubMenu::~SignalPathControlSubMenu() {
	delete sp;
}

const std::string& SignalPathControlSubMenu::SubMenuDescription() const
{
	return desc;
}

void SignalPathControlSubMenu::SubMenuPrint()
{
	std::cout << std::endl << std::endl;
	std::cout << " ******** SIGNAL PATH CONTROL MENU ********" << std::endl;
	std::cout << " 1) Set DDS Power" << std::endl;
	std::cout << " 2) Set RF Gain Wiper #1" << std::endl;
	std::cout << " 3) Set RF Gain Wiper #2" << std::endl;
	std::cout << " 4) Select RF Gain Control Source" << std::endl;
	std::cout << " 5) Toggle Image Path Amplitude Compensation" << std::endl;
	std::cout << " 6) Toggle Image Path Phase Compensation" << std::endl;
	std::cout << " 7) Toggle X/Y Phase Compensation Mode" << std::endl;
	std::cout << " q) Back to Main Menu" << std::endl;
}

void SignalPathControlSubMenu::LaunchSubMenu()
{
	std::string s;
	double d;

	do {
		this->SubMenuPrint();
		std::getline( std::cin, s);
		if ((s.find("q") != std::string::npos) || (s.find("Q") != std::string::npos)) {
			break;
		} else {
			int index;
			std::istringstream iss(s);
			iss >> index;
			if (!iss.fail()) {
				if ((index > 0) && (index <= num_submenu_items)) {
					switch(index) {
					case 1: {
						// Set DDS Power
						std::cout << "Enter DDS Power(%): ";
						std::getline( std::cin, s);
						iss.clear();
						iss.str(s);
						iss >> d;
						if (!iss.fail()) {
							if (sp->UpdateDDSPowerLevel(iMS::Percent(d)))
								std::cout << "DDS Power set to " << d << "%" << std::endl;
							else
								std::cout << "Operation Unsuccessful" << std::endl;
						}
					}
					break;
					case 2: {
						// Set RF Gain Wiper #1
						std::cout << "Enter RF Gain Wiper 1(%): ";
						std::getline( std::cin, s);
						iss.clear();
						iss.str(s);
						iss >> d;
						if (!iss.fail()) {
							if (sp->UpdateRFAmplitude(iMS::SignalPath::AmplitudeControl::WIPER_1, iMS::Percent(d)))
								std::cout << "RF Gain Wiper #1 set to " << d << "%" << std::endl;
							else
								std::cout << "Operation Unsuccessful" << std::endl;
						}
					}
					break;
					case 3: {
						// Set RF Gain Wiper #2
						std::cout << "Enter RF Gain Wiper 2(%): ";
						std::getline( std::cin, s);
						iss.clear();
						iss.str(s);
						iss >> d;
						if (!iss.fail()) {
							if (sp->UpdateRFAmplitude(iMS::SignalPath::AmplitudeControl::WIPER_2, iMS::Percent(d)))
								std::cout << "RF Gain Wiper #2 set to " << d << "%" << std::endl;
							else
								std::cout << "Operation Unsuccessful" << std::endl;
						}
					}
					break;
					case 4: {
						// Select RF Gain Control Source
						std::cout << "Select Source: (1) = None (RF Off), (2) = External modulation inputs," << std::endl <<
								     "               (3) = RF Gain Wiper #1, (4) = RF Gain Wiper #1 " << std::endl;
						std::getline( std::cin, s);
						iss.clear();
						iss.str(s);
						iss >> index;
						if ((index >= 1) && (index <= 4)) {
							switch(index) {
							case 1: {
								sp->SwitchRFAmplitudeControlSource(iMS::SignalPath::AmplitudeControl::OFF);
								std::cout << "RF Amplitude Control now OFF" << std::endl;
								break;
							}
							case 2: {
								sp->SwitchRFAmplitudeControlSource(iMS::SignalPath::AmplitudeControl::EXTERNAL);
								std::cout << "RF Amplitude Control now EXTERNAL" << std::endl;
								break;
							}
							case 3: {
								sp->SwitchRFAmplitudeControlSource(iMS::SignalPath::AmplitudeControl::WIPER_1);
								std::cout << "RF Amplitude Control now WIPER 1" << std::endl;
								break;
							}
							case 4: {
								sp->SwitchRFAmplitudeControlSource(iMS::SignalPath::AmplitudeControl::WIPER_2);
								std::cout << "RF Amplitude Control now WIPER 2" << std::endl;
								break;
							}
							}
						} else {
							std::cout << "Control source unchanged" << std::endl;
						}
					}
					break;
					case 5: {
						// Toggle Image Path Amplitude Compensation
						if (this->amplComp == iMS::SignalPath::Compensation::ACTIVE) {
							this->amplComp = iMS::SignalPath::Compensation::BYPASS;
						} else {
							this->amplComp = iMS::SignalPath::Compensation::ACTIVE;
						}
						if (sp->EnableImagePathCompensation(this->amplComp, this->phaseComp)) {
							if (this->amplComp == iMS::SignalPath::Compensation::ACTIVE)
								std::cout << "Amplitude Compensation ACTIVE" << std::endl;
							else
								std::cout << "Amplitude Compensation BYPASSED" << std::endl;
						} else {
							std::cout << "Failed to set Amplitude Compensation" << std::endl;
						}
					}
					break;
					case 6: {
						// Toggle Image Path Phase Compensation
						if (this->phaseComp == iMS::SignalPath::Compensation::ACTIVE) {
							this->phaseComp = iMS::SignalPath::Compensation::BYPASS;
						} else {
							this->phaseComp = iMS::SignalPath::Compensation::ACTIVE;
						}
						if (sp->EnableImagePathCompensation(this->amplComp, this->phaseComp)) {
							if (this->phaseComp == iMS::SignalPath::Compensation::ACTIVE)
								std::cout << "Phase Compensation ACTIVE" << std::endl;
							else
								std::cout << "Phase Compensation BYPASSED" << std::endl;
						} else {
							std::cout << "Failed to set Phase Compensation" << std::endl;
						}
					}
					break;
					case 7: {
						// Toggle X/Y Phase Compensation Mode
						this->XYMode = !this->XYMode;
						if (sp->EnableXYPhaseCompensation(this->XYMode)) {
							if (this->XYMode)
								std::cout << "X/Y Channel Mode ENABLED" << std::endl;
							else
								std::cout << "X/Y Channel Mode DISABLED" << std::endl;
						} else {
							std::cout << "Failed to set XY Mode" << std::endl;
						}
					}
					break;
					}
				}
			}
		}
	} while(1);

}



