/*-----------------------------------------------------------------------------
/ Title      : iMS Test Program FileSystem Sub-Menu Source
/ Project    : Isomet Modular Synthesiser System
/------------------------------------------------------------------------------
/ File       : $URL: http://nutmeg/svn/sw/trunk/09-Isomet/iMS_SDK/console_app/SubMenus/SystemSubMenu.cpp $
/ Author     : $Author: dave $
/ Company    : Isomet (UK) Ltd
/ Created    : 2017-09-06
/ Last update: $Date: 2017-09-11 23:55:34 +0100 (Mon, 11 Sep 2017) $
/ Platform   :
/ Standard   : C++11
/ Revision   : $Rev: 300 $
/------------------------------------------------------------------------------
/ Description:
/------------------------------------------------------------------------------
/ Copyright (c) 2017 Isomet (UK) Ltd. All Rights Reserved.
/------------------------------------------------------------------------------
/ Revisions  :
/ Date        Version  Author  Description
/ 2017-09-06  1.0      dc      Created
/
/----------------------------------------------------------------------------*/

#include "SystemSubMenu.h"

#include <iostream>
#include <sstream>
#include <fstream>
#include <chrono>

static std::string desc("System Functions Menu");

class SystemFuncSupervisor : public iMS::IEventHandler
{
private:
	std::atomic<double> temp1 {-999.0};
	std::atomic<double> temp2 {-999.0};
public:
	void EventAction(void* sender, const int message, const double param)
	{
		switch (message)
		{
		case (iMS::SystemFuncEvents::SYNTH_TEMPERATURE_1) : temp1.store(param); break;
		case (iMS::SystemFuncEvents::SYNTH_TEMPERATURE_2) : temp2.store(param); break;
		}
	}
	void Report() {
		std::cout << "System Temperature #1: " << temp1.load() << "C #2: " << temp2.load() << "C      " ;
	}
};

const int SystemSubMenu::num_submenu_items = 5;

SystemSubMenu::SystemSubMenu(iMS::IMSSystem& ims) :
			thisIMS(ims),
			sys(new iMS::SystemFunc(ims))
{
	chan12 = chan34 = false;
}

SystemSubMenu::~SystemSubMenu() {
	delete sys;
}

const std::string& SystemSubMenu::SubMenuDescription() const
{
	return desc;
}

void SystemSubMenu::SubMenuPrint()
{
	std::cout << std::endl << std::endl;
	std::cout << " ******** SYSTEM FUNCTIONS MENU ********" << std::endl;
	std::cout << " 1) Enable RF Amplifier" << std::endl;
	std::cout << " 2) Enable RF Channels 1 & 2" << std::endl;
	std::cout << " 3) Enable RF Channels 3 & 4" << std::endl;
	std::cout << " 4) Disable all RF" << std::endl;
	std::cout << " 5) Monitor System Temperature" << std::endl;
	std::cout << " q) Back to Main Menu" << std::endl;
}

void SystemSubMenu::LaunchSubMenu()
{
	std::string s;

	do {
		this->SubMenuPrint();
		std::getline( std::cin, s);
		if ((s.find("q") != std::string::npos) || (s.find("Q") != std::string::npos)) {
			break;
		} else {
			int index;
			std::istringstream iss(s);
			iss >> index;
			if (!iss.fail()) {
				if ((index > 0) && (index <= num_submenu_items)) {
					switch(index) {
					case 1: {
						// Enable RF Amplifier
						sys->EnableAmplifier(true);
						std::cout << "RF Amplifier ENABLED" << std::endl;
						break;
					}
					case 2: {
						// Enable RF Channels 1 & 2
						sys->EnableRFChannels(true, chan34);
						chan12 = true;
						std::cout << "RF Channels 1 & 2 ENABLED" << std::endl;
						break;
					}
					case 3: {
						// Enable RF Channels 1 & 2
						sys->EnableRFChannels(chan12, true);
						chan34 = true;
						std::cout << "RF Channels 3 & 4 ENABLED" << std::endl;
						break;
					}
					case 4: {
						// Disable all RF
						sys->EnableAmplifier(false);
						sys->EnableRFChannels(false, false);
						chan12 = chan34 = false;
						std::cout << "All RF DISABLED" << std::endl;
						break;
					}
					case 5: {
						// Monitor System Temperature
						std::cout << "Press Enter to stop\n";

						iMS::Auxiliary aux(thisIMS);
						aux.AssignLED(iMS::Auxiliary::LED_SINK::YELLOW, iMS::Auxiliary::LED_SOURCE::OVERTEMP);

						std::thread tempThread(&SystemSubMenu::ReadTemperature, this);

						std::getline( std::cin, s);

						tempThreadRunning.store(false);  // must set this to cancel threads
						tempThread.join();

						break;
					}

					}
				}
			}
		}
	} while(1);

}

void SystemSubMenu::ReadTemperature()
{
	SystemFuncSupervisor sfs;
	sys->SystemFuncEventSubscribe(iMS::SystemFuncEvents::SYNTH_TEMPERATURE_1, &sfs);
	sys->SystemFuncEventSubscribe(iMS::SystemFuncEvents::SYNTH_TEMPERATURE_2, &sfs);
	tempThreadRunning.store(true);

	do {
		sfs.Report();
		std::cout << "\r";  // Carriage return to overwrite existing line with next reading
		std::this_thread::sleep_for(std::chrono::milliseconds(250));
		sys->ReadSystemTemperature(iMS::SystemFunc::TemperatureSensor::TEMP_SENSOR_1);
		sys->ReadSystemTemperature(iMS::SystemFunc::TemperatureSensor::TEMP_SENSOR_2);
	} while(this->tempThreadRunning.load());

	sys->SystemFuncEventUnsubscribe(iMS::SystemFuncEvents::SYNTH_TEMPERATURE_1, &sfs);
	sys->SystemFuncEventUnsubscribe(iMS::SystemFuncEvents::SYNTH_TEMPERATURE_2, &sfs);
}







